import globalPluginHandler
import addonHandler
import gui
import os
import wx
from logHandler import log
import config
import api
import vision
from locationHelper import RectLTRB
try:
	import vision
except ImportError:
	raise RuntimeError("No vision framework support")

#We need to initialize translation and localization support:
addonHandler.initTranslation()

class GlobalPlugin(globalPluginHandler.GlobalPlugin):
	"""
	USA globalPlugin class of which an instance is created, i.e. upon loading of NVDA.
	This is used to initialize USA Remote if applicable. Also it sets the help file direction in the settings.
	For the Object Reader we also need the mouseMove event.
	"""

	def __init__(self):
		super().__init__()
		self.remoteHandler = None
		wx.CallAfter(self.initializeRemote)
		self.help_item = gui.mainFrame.sysTrayIcon.helpMenu.Append(wx.ID_ANY, _("&Ultimate Screen Access help"), _("Manual for Ultimate Screen Access"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, lambda e : os.startfile(addonHandler.getCodeAddon().getDocFilePath()), self.help_item)

		log.info("USA: plugin initialized")

	def terminate(self):
		if self.remoteHandler:
			self.remoteHandler.terminate()
			self.remoteHandler = None
		gui.mainFrame.sysTrayIcon.helpMenu.Remove(self.help_item)
		self.help_item.Destroy()
		self.help_item=None

		super().terminate()

	def initializeRemote(self):
		try:
			self.remoteHandler = RemoteHandler()
		except (NotImplementedError, RuntimeError):
			log.info("USA: USA remote capabilities not available")
			self.remoteHandler = None
		else:
			log.info("USA: USA remote capabilities enabled")

class RemoteHandler(object):

	def __init__(self):
		self.connectionType = None
		self.transport = None
		# check if unicorn is running
		try:
			self.remoteClient = next(p for p in globalPluginHandler.runningPlugins if p.__module__=='globalPlugins.unicorn')
		except StopIteration:
			raise RuntimeError
		# register callbacks
		self.remoteClient.callback_manager.register_callback('transport_connect',self.onTransportConnected)
		self.remoteClient.callback_manager.register_callback('transport_disconnect',self.onTransportDisconnected)

		if self.remoteClient.slave_transport:
			self.onTransportConnected(connection_type='slave',transport=self.remoteClient.slave_transport)
		elif self.remoteClient.master_transport:
			self.onTransportConnected(connection_type='master',transport=self.remoteClient.master_transport)

	def sendVisionCommandOverUnicorn(self, command, **kwargs):
		if self.transport:
			self.transport.send(type="vision", command=command, **kwargs)
		else:
			log.error("Unicorn transport for USA not initialized.")

	def terminate(self):
		self.unregisterRemoteTransport()
		self.remoteClient.callback_manager.unregister_callback('transport_connect',self.onTransportConnected)
		self.remoteClient.callback_manager.unregister_callback('transport_disconnect',self.onTransportDisconnected)
		self.remoteClient = None

	def unregisterRemoteTransport(self):
		if self.transport:
			try:
				self.transport.callback_manager.unregister_callback('msg_vision', self.handleVisionPacket)
			except ValueError:
				pass

	def onTransportConnected(self,connection_type,transport,*args,**kwargs):
		transport.callback_manager.register_callback('msg_vision', self.handleVisionPacket)
		self.connectionType = connection_type
		self.transport = transport

	def onTransportDisconnected(self,connection_type,*args,**kwargs):
		# We do not have to unregister callbacks here
		self.connectionType = None
		self.transport = None

	def handleVisionPacket(self,command, *args,**kwargs):
		if not vision.handler:
			return
		try:
			usa = vision.handler.getProviderInstance(vision.handler.getProviderInfo("usa"))
		except LookupError:
			usa = None
		if not usa:
			return
		commandMapping = usa.getUnicornCommandMapping()
		kwargs = usa.deserializeUnicornArguments(command, **kwargs)
		log.info(f"{command} {kwargs}")
		try:
			if kwargs:
				commandMapping[command](**kwargs)
			else:
				commandMapping[command]()
		except Exception as e:
			log.error(f"An error occured when handling a vision package for USA: \n {e}")
