

from comtypes.client import CoGetObject
from ctypes import *
from ctypes.wintypes import *
from hashlib import sha1
import time
import os
import os.path
import winreg
import sys
import wx
import json

if getattr(sys, 'frozen', False):
	# if frozen, get embeded file
	cacert = os.path.join(os.path.dirname(sys.executable), 'cacert.pem')
	os.environ["REQUESTS_CA_BUNDLE"] = cacert


import json
import typing
import urllib.error
import urllib.parse
import urllib.request
import ssl
from email.message import Message

from logHandler import log

class Response(typing.NamedTuple):
	body: str
	headers: Message
	status: int
	error_count: int = 0

	def json(self) -> typing.Any:
		"""
		Decode body's JSON.

		Returns:
			Pythonic representation of the JSON object
		"""
		try:
			output = json.loads(self.body)
		except json.JSONDecodeError:
			output = ""
		return output


def request(
	url: str,
	data: dict = None,
	params: dict = None,
	headers: dict = None,
	method: str = "GET",
	data_as_json: bool = True,
	error_count: int = 0,
) -> Response:
	if not url.casefold().startswith("http"):
		raise urllib.error.URLError("Incorrect and possibly insecure protocol in url")
	method = method.upper()
	request_data = None
	headers = headers or {}
	data = data or {}
	params = params or {}
	headers = {"Accept": "application/json", **headers}

	if method == "GET":
		params = {**params, **data}
		data = None

	if params:
		url += "?" + urllib.parse.urlencode(params, doseq=True, safe="/")

	if data:
		if data_as_json:
			request_data = json.dumps(data).encode()
			headers["Content-Type"] = "application/json; charset=UTF-8"
		else:
			request_data = urllib.parse.urlencode(data).encode()

	httprequest = urllib.request.Request(
		url, data=request_data, headers=headers, method=method
	)
	
	try:
		context = ssl._create_unverified_context()
		with urllib.request.urlopen(httprequest, context=context) as httpresponse:
			print(httpresponse.headers)
			response = Response(
				headers=httpresponse.headers,
				status=httpresponse.status,
				body=httpresponse.read().decode(
					httpresponse.headers.get_content_charset("utf-8")
				),
			)
	except urllib.error.HTTPError as e:
		response = Response(
			body=str(e.reason),
			headers=e.headers,
			status=e.code,
			error_count=error_count + 1,
		)

	return response


class LicenseManager:
	server = "https://access4u.eu/"

	def __init__(self, productId, productName):
		super().__init__()
		self.productId = productId
		self.productName = productName
		self.licenseKey = None
		self.activationHash = None
		self.timeStamp = None
		self.regPath = os.path.join("SOFTWARE", "ACCESS4U", productName)

	def loadFromRegistry(self):
		with winreg.OpenKeyEx(
				winreg.HKEY_CURRENT_USER,
				self.regPath,
				0,
				winreg.KEY_READ | winreg.KEY_WOW64_32KEY
		) as key:
			self.licenseKey = winreg.QueryValueEx(key, "LicId")[0]
			hash = winreg.QueryValueEx(key, "AcId")[0]
			self.activationHash = hash[8:]
			self.timeStamp = int(hash[:8], 16)
			log.info("still in loadFromRegistry ")
			if self.calculateActivationHash(self.timeStamp) != self.activationHash :
				log.info("raise error")
				self.licenseKey = self.activationHash = self.timeStamp = None
				raise ValueError("Invalid activation info")
		log.info("end of loadFromRegistry")


	def obtainValidLicense(self):
		try:
			self.loadFromRegistry()
		except ValueError:
			self.wipeRegistry()
			raise
		timeDifference = time.time() - self.timeStamp

		if timeDifference >= 60 * 60 * 24 * 30:
			# Check validity
			try:
				res = self.verifyActivation(self.licenseKey)
			except RuntimeError:
				raise RuntimeError("License is no longer valid and can't be verified")
			if res:
				self.saveToRegistry()
				return res
			else:
				raise RuntimeError("Activation seemed inactivated after recheck")
		else:
			# we have a license that is checked this month
			return True

	def activate(self, licenseKey, hardwareId=None):

		if not hardwareId: hardwareId = self.getHardwareId()
		self.licenseKey = licenseKey
		# Check validity
		try:
			res = self.verifyActivation(licenseKey)
		except RuntimeError:
			raise RuntimeError("License is no longer valid and can't be verified")

		if res:
			self.timeStamp = int(time.time())
			self.saveToRegistry()
			return
		uri = {
			"api_key": licenseKey,
			"request": "activate",
		}

		# The caller should catch errors.
		data = self.getDataFromActivationServer(uri, hardwareId=hardwareId)

		if not data.get("success"):
			raise RuntimeError(data['error'])

		activated = data.get("activated")
		if activated:
			message = data['message']
			self.licenseKey = licenseKey
			self.timeStamp = int(time.time())
			self.saveToRegistry()
			return message
		if not data.get("success"):
			raise RuntimeError(data['error'])
		raise RuntimeError("Activation failed")

	def deactivate(self, hardwareId=None):
		log.info("load from registery")
		self.loadFromRegistry()
		log.info("loaded from registery")
		if not hardwareId: hardwareId = self.getHardwareId()

		uri = {
			"api_key": self.licenseKey,
			"request": "deactivate",
		}
		# The caller should catch errors.
		data = self.getDataFromActivationServer(uri, hardwareId=hardwareId)
		log.info(f"data {data}")
		if not data.get("success"):
			raise RuntimeError(data['error'])
		deactivated = data.get("deactivated")
		if deactivated:
			message = data['activations_remaining']
			self.wipeRegistry()
			return message
		raise RuntimeError("Deactivation failed")

	def verifyActivation(self, licenseKey, hardwareId=None):
		if not hardwareId: hardwareId = self.getHardwareId()
		res = False
		uri = {
			"api_key": licenseKey if licenseKey else self.licenseKey,
			"request": "status",
		}
		# The caller should catch errors.
		data = self.getDataFromActivationServer(uri, hardwareId=hardwareId)
		if not data.get("success"):
			raise RuntimeError(data['error'])
		activated = data.get("status_check")
		if not activated:
			raise RuntimeError("Status check failed")
		else:
			res = activated == "active"
		return res

	@classmethod
	def getHardwareId(cls):
		serialNum = DWORD()
		windll.kernel32.GetVolumeInformationW("c:\\", None, 0, byref(serialNum), None, None, None, 0)
		wmi = CoGetObject("winmgmts:", dynamic=True)
		uuid = next(i.Properties_["uuid"].Value for i in wmi.InstancesOf("win32_computersystemproduct"))
		id = str(serialNum.value) + uuid
		return cls.getHash(id)

	@classmethod
	def getHash(cls, raw):
		if isinstance(raw, str):
			raw = raw.encode("mbcs")
		return sha1(raw).hexdigest().upper()

	def calculateActivationHash(self, timeStamp, licenseKey=None, hardwareId=None):
		log.info("calculateActivationHash")
		if licenseKey is None:
			licenseKey = self.licenseKey
		if not hardwareId:
			hardwareId = self.getHardwareId()
		raw = self.productName + hardwareId + licenseKey + str(timeStamp)

		lst = list(raw)
		for i in range(0, len(raw) // 2, 2):
			lst[i], lst[len(raw) - 1 - i] = lst[len(raw) - 1 - i], lst[i]
		processed = "".join(lst)
		hash = self.getHash(processed)
		log.info(f"has {hash}")
		return hash

	def getDataFromActivationServer(self, uri={}, hardwareId=None):
		headers = {"Accept": "application/json"}
		uri.update({
			"wc-api": "wc-am-api",
			"product_id": self.productId,
			"object": os.environ['COMPUTERNAME'],
			"instance": hardwareId or self.getHardwareId(),
		})

		res =  request(
				method="post",
				url=self.server,
				params=uri,
				headers=headers,
		)
		return res.json()

	def saveToRegistry(self):
		with winreg.CreateKeyEx(
				winreg.HKEY_CURRENT_USER,
				self.regPath,
				0,
				winreg.KEY_WRITE | winreg.KEY_WOW64_32KEY
		) as key:
			winreg.SetValueEx(key, "LicId", 0, winreg.REG_SZ, self.licenseKey)
			hash = hex(self.timeStamp)[2:].upper() + self.calculateActivationHash(self.timeStamp)
			winreg.SetValueEx(key, "AcId", 0, winreg.REG_SZ, hash)

	def wipeRegistry(self):
		with winreg.OpenKeyEx(
				winreg.HKEY_CURRENT_USER,
				self.regPath,
				0,
				winreg.KEY_SET_VALUE | winreg.KEY_WOW64_32KEY
		) as key:
			try:
				winreg.DeleteValue(key, "LicId")
			except FileNotFoundError:
				pass
			try:
				winreg.DeleteValue(key, "AcId")
			except FileNotFoundError:
				pass
		self.licenseKey = self.activationHash = self.timeStamp = None


import urllib.request
import json
from . import update
def deactivateUSA():
	log.info(update.channels['latest'])
	context = ssl._create_unverified_context()
	with urllib.request.urlopen(update.channels['latest'], context = context) as url:
		data = json.loads(url.read().decode())
	success = False
	log.info(data)
	products = data['products']
	for ii in products:
		if ii['name'] == 'USA':

			for jj in ii['versions']:
				try:
					log.info(f"try to deactivate {jj['id']}")
					lm = LicenseManager(jj['id'], 'USA')
					message = lm.deactivate()
					success = True
					return success, message
				except Exception as e :
					log.info(e)

	return success, "failed to deactivate known USA versions"



