
"""
2021, Babbage B.V.
Extension to the USA add-on to let USA follow spoken words with a SayAll that starts where the mouse is.
"""
import speech
import winUser
import wx
from logHandler import log
from mouseHandler import *
import api
import speech.sayAll
import globalCommands
import time
import virtualBuffers
import controlTypes
import NVDAObjects
import textInfos


class objectReader:
	"""
	The object reader Mode turns off speech by default. With this you can let NVDA read whatever is under the mouse with a
	simple command.
	"""
	def __init__(self, USAinstance):
		self.USA = USAinstance
		self.enabled = False

		if self.USA.getSettings().objectReaderMode:
			self.enable()

		self.isSelecting = False
		self.oldFollowMouseValue = False
		self.oldenableMouseTracking = False
		self.vbuf = None


	def terminate(self):
		if self.enabled:
			self.disable()


	def enable(self):
		log.info("USA: enable object Reader Mode")
		speech.speech.setSpeechMode(speech.speech.SpeechMode.off)
		self.enabled = True

	def disable(self):
		log.info("USA: disable object Reader Mode")
		self.enabled = False

	def handleForegroundChange(self, obj):
		pass

	def handleReviewMove(self, context):
		pass

	def handleFocusChange(self, obj):
		pass




	def callAfterCommandsForMouseRead(self):
		speech.speech.setSpeechMode(speech.speech.SpeechMode.talk)

		wx.CallLater(100, speech.sayAll.SayAllHandler.readText, speech.sayAll.CURSOR.CARET)
		wx.CallLater(200, self.checkIfSayAllIsRunning)
		wx.CallLater(500, self.disableVbuf)

	def reloadVbuf(self):
		desktop = api.getDesktopObject()
		x, y = winUser.getCursorPos()
		obj = desktop.objectFromPoint(x, y)
		self.vbuf = self.setCaretOnRelevantTextInfo(obj)
		log.info(f"USA: objectReader; self.vbuf: {self.vbuf}")

	def firstVbufLoad(self):
		log.info("USA: objectReader; make vbuf")
		globalCommands.commands.script_toggleVirtualBufferPassThrough("")
		wx.CallLater(500, self.reloadVbuf)
		wx.CallLater(700, self.callAfterCommandsForMouseRead)

	def readAllUnderMouse(self):
		x, y = winUser.getCursorPos()
		desktop = api.getDesktopObject()

		obj = desktop.objectFromPoint(x, y)

		vbufClass = self.getTIClass(obj)
		if vbufClass and issubclass(vbufClass, (virtualBuffers.VirtualBuffer,) ):
			vbuf = obj.treeInterceptor

			if not vbuf:
				# the vbuf needs to be made first. Everything else needs to be called after. Thus we return early here
				self.firstVbufLoad()
				return
			else:
				if vbuf.passThrough:
					vbuf.passThrough = False
					vbuf.disableAutoPassThrough = False

			#vbuf.passThrough = False
			self.reloadVbuf()

		self.callAfterCommandsForMouseRead()




	def disableVbuf(self):
		if self.vbuf:
			self.vbuf.passThrough = False
			self.vbuf.disableAutoPassThrough = False
		#self.vbuf = None

	def setCaretOnRelevantTextInfo(self, obj):

		vbuf = obj.treeInterceptor
		log.info(f"USA: ObjectReader; obj {obj} vbuf {vbuf}")
		try:
			info = vbuf.makeTextInfo(obj)

			info.updateCaret()
			log.info(f"made info object {info} \n caret object: {api.getCaretObject()}")
			return vbuf
		except:
			parent = obj.parent
			if parent:
				return self.setCaretOnRelevantTextInfo(parent)


	def checkIfSayAllIsRunning(self):
		if speech.sayAll.SayAllHandler.isRunning():
			wx.CallLater(100, self.checkIfSayAllIsRunning)
		else:
			speech.speech.setSpeechMode(speech.speech.SpeechMode.off)
			self.USA.wordFollower.rectHighlighter.updateCustomRect(rect=None)
			log.info(f"checkIfSayAllIsRunning stopped {self.vbuf}")
			if self.vbuf:
				log.info("disabling vbuf passThrough")
				self.vbuf.passThrough = True
				self.vbuf.disableAutoPassThrough = True


	def getTIClass(self, obj):
		# gets the first Tree Interceptor of the object, if there is any
		vbufClass = None
		while not vbufClass:

			if not obj.role == controlTypes.Role.WINDOW:
				try:
					vbufClass = obj.treeInterceptorClass
				except:
					vbufClass = None
				obj = obj.parent
			else:
				return None

		return vbufClass

	def toggleVBuf(self, obj):

		vBufClass = self.getTIClass(obj)
		vbuf = None
		if vBufClass:

			if issubclass(vBufClass, (virtualBuffers.VirtualBuffer,) ):
				log.info("USA: objectReader; activate vbuf")
				vbuf = self.activateVBuf(obj)
			else:
				log.info("USA: objectReader; deactivate vbuf")
				vbuf = self.deactivateVBuf(obj)

		return vbuf

	# def activateVBuf(self, obj):
	# 	vbuf = obj.treeInterceptor
	# 	if not vbuf:
	# 		globalCommands.commands.script_toggleVirtualBufferPassThrough("")
	# 		vbuf = obj.treeInterceptor
	#
	# 	else:
	# 		if vbuf.passThrough:
	# 			vbuf.passThrough = not vbuf.passThrough
	#
	# 	if not vbuf:
	# 		log.error("USA: objectReader; no vbuf could be created due to unknown reason")
	# 	return vbuf
	#
	# def deactivateVBuf(self, obj):
	# 	vbuf = obj.treeInterceptor
	# 	if vbuf:
	# 		vbuf.passThrough = True
	# 	return vbuf
	#
	# def getMouseObject(self):
	# 	x, y = winUser.getCursorPos()
	# 	mouseHandler.executeMouseMoveEvent(x, y)
	# 	obj = api.getMouseObject()
	# 	return obj, x, y
	#
	#
	# def makeTextInfo(self, obj, x, y, vbuf):
	# 	# makes a textInfo with either the vBuf or the object itself
	# 	if vbuf:
	# 		info = vbuf.makeTextInfo(obj)
	# 	else:
	# 		try:
	#
	# 			info=obj.makeTextInfo(locationHelper.Point(x,y))
	# 		except NotImplementedError:
	# 			log.error("could not make textInfo object the default way")
	# 			info=NVDAObjectTextInfo(obj,textInfos.POSITION_FIRST)
	#
	# 	return info
	#
	# def sayAllOverlay(self):
	# 	obj, x, y = self.getMouseObject()
	# 	log.info(f"obj: {obj}")
	# 	vbuf = self.toggleVBuf(obj)
	# 	wx.CallAfter(self.sayAllOverlay2)
	#
	# def sayAllOverlay2(self):
	# 	obj, x, y = self.getMouseObject()
	# 	vbuf = obj.treeInterceptor
	# 	info = self.makeTextInfo(obj, x, y, vbuf)
	#
	# 	api.setReviewPosition(info)
	# 	try:
	# 		info.updateCaret()
	# 	except:
	# 		log.error("could not move caret to review position")
	#
	# 	log.info(
	# 		f" vbuf object: {vbuf} \n caret object: {api.getCaretObject()} \n mouse object: {obj} \n info obj {info}")
	# 	speech.speech.setSpeechMode(speech.speech.SpeechMode.talk)
	# 	wx.CallLater(100, self.checkIfSayAllIsRunning)
	# 	if vbuf and not vbuf.passThrough:
	# 		log.info("reading from review")
	# 		wx.CallLater(100, speech.sayAll.SayAllHandler.readText, speech.sayAll.CURSOR.REVIEW)
	# 	else:
	# 		log.info("reading from caret")
	# 		#reading from caret can start immediately without problems
	# 		speech.sayAll.SayAllHandler.readText(speech.sayAll.CURSOR.CARET)



#
# class BrowseModePanelOverlay(gui.settingsDialogs.BrowseModePanel):
# 	def makeSettings(self, settingsSizer):
# 		super().makeSettings(settingsSizer)
# 		self.enableOnPageLoadCheckBox.Disable()
# 		self.autoSayAllCheckBox.Disable()
#
# class ReviewCursorPanelOverlay(gui.settingsDialogs.ReviewCursorPanel):
# 	def makeSettings(self, settingsSizer):
# 		super().makeSettings(settingsSizer)
# 		self.followMouseCheckBox.Disable()
#
# class MouseSettingsPanelOverlay(gui.settingsDialogs.MouseSettingsPanel):
# 	def makeSettings(self, settingsSizer):
# 		super().makeSettings(settingsSizer)
# 		self.mouseTrackingCheckBox.Disable()



